<?php
// $Id: Shell.php.modify.User
// Copyright (c) 2008-2024 Michael Stauber, SOLARSPEED.NET
// Copyright (c) 2008-2024 Team BlueOnyx, BLUEONYX.IT
// Copyright (c) 2001 Sun Microsystems, Inc.
// All Rights Reserved.

include_once('UIExtension.php');

class ShellExtension extends UIExtension {
    function display(&$ServerScriptHelper, &$container, $parameters) {

        $CI =& get_instance();
        $BX_SESSION = $CI->getBX_SESSION();
        $System = $CI->getSystem();

        $factory = $ServerScriptHelper->getHtmlComponentFactory("base-shell");
        $BxPage = $factory->getPage();
        $cce = $ServerScriptHelper->getCceClient();
        $i18n = new I18n("base-shell", $BX_SESSION['loginUser']['localePreference']);

        $System_SSH = $cce->get($System['OID'], 'SSH');

        $Shell_enabled_Map = 
            array(
                "0" => "none", 
                "1" => "jailed_sftp_scp_rsync", 
                "2" => "jailed_shell", 
                "3" => "full_shell"
            );

        // if configuring for virtual sites, don't show if the feature
        // is not enabled 
        $access = '1';
        if ( isset($parameters["VSITE_OID"]) ) {
            $feature = $cce->get($parameters["VSITE_OID"], "Shell");
            if ($feature["enabled"] == '0') {
                $access = '0';
            }
            elseif ($feature["enabled"] == '1') {
                unset($Shell_enabled_Map["2"]);
                unset($Shell_enabled_Map["3"]);
            }
            elseif ($feature["enabled"] == '2') {
                unset($Shell_enabled_Map["3"]);
            }
        }

        $UserData = $cce->get($parameters["CCE_OID"]);
        $User_Shell = $cce->get($parameters["CCE_OID"], "Shell");
        $User_SSH = $cce->get($parameters["CCE_OID"], "SSH");

        //
        //-- Sense Checks:
        //

        if ($System_SSH['GoogleAuthentication'] == '0') {
            // System has 'GoogleAuthentication' OFF, User cannot have it ON:
            $User_SSH['GoogleAuthentication'] = '0';
        }

        if (isset($feature['GoogleAuthentication'])) {
            if ($feature['GoogleAuthentication'] == '0') {
                // Vsite has 'GoogleAuthentication' OFF, User cannot have it ON:
                $User_SSH['GoogleAuthentication'] = '0';
            }
        }

        // Find out if user has the Google Authenticator config and QR image in his homedir:
        $is_there = '';
        $loginName = $UserData['name'];
        $dotgoogle_authenticator = '0';
        $ret = $CI->serverScriptHelper->shell("/usr/sausalito/sbin/gauth_check.sh $loginName" . $is_there, $output, 'root', $BX_SESSION['sessionId']);
        if ($ret == 0) {
            // Google Authenticator Config present:
            $dotgoogle_authenticator = '1';
        }

        if (($User_SSH['GoogleAuthentication'] == '0') && ($dotgoogle_authenticator == '1')) {
            // User has 'GoogleAuthentication' disabled. 
            // User has dotgoogle_authenticator present
            // Fix it:
            $cce->set($parameters["CCE_OID"], 'SSH', array('GoogleAuthentication' => '1'));
            $cce->set($parameters["CCE_OID"], 'SSH', array('GoogleAuthentication' => '0'));
            $User_SSH['GoogleAuthentication'] = '0';
        }
        if (($User_SSH['GoogleAuthentication'] == '1') && ($dotgoogle_authenticator == '0')) {
            // User has 'GoogleAuthentication' enabled.
            // User has dotgoogle_authenticator absent
            // Fix it:
            $cce->set($parameters["CCE_OID"], 'SSH', array('GoogleAuthentication' => '0'));
            $cce->set($parameters["CCE_OID"], 'SSH', array('GoogleAuthentication' => '1'));
            $User_SSH['GoogleAuthentication'] = '1';
        }

        //
        //-- Page:
        //

        // assume PagedBlock because that is most likely going to be it
        $pageIds = $container->getPageIds();

        // Enable Shell Access:
        if ($access) {
            $shellEnable = $factory->getMultiChoice("Shell_enabled", array_values($Shell_enabled_Map));
            $shellEnable->setSelected($Shell_enabled_Map[$User_Shell['enabled']], true);
            $container->addFormField($shellEnable, $factory->getLabel("enableShell"), $pageIds[0]);
        }
        else {
            $word = $Shell_enabled_Map[$User_Shell['enabled']];
            $nokey_info = $i18n->getClean("[[base-shell.$word]]", false, array());
            $xxx = $factory->getTextField("Shell_enabled_Dummy", $nokey_info , 'r');
            $container->addFormField(
                $xxx,
                $factory->getLabel("enableShell"),
                $pageIds[0]
            );

            $xxx = $factory->getTextField("Shell_enabled", $User_Shell['enabled'] , '');
            $container->addFormField(
                $xxx,
                $factory->getLabel("enableShell"),
                $pageIds[0]
            );
        }

        // Google Authenticator enabled or not:
        $GA_access = 'rw';
        if (($System_SSH['GoogleAuthentication'] == '0') || ($feature['GoogleAuthentication'] == '0')) {
            // 'GoogleAuthentication' is disabled in the server settings or Vsite settings:
            $User_SSH['GoogleAuthentication'] = '0';
            $GA_access = 'r';
        }

        $ga_is_enabled = '0';
        if (($User_SSH['GoogleAuthentication'] == '1') && ($dotgoogle_authenticator == '1')) {
            $ga_is_enabled = '1';

            // Find out if user has the Google Authenticator config and QR image in his homedir:
            $is_there = '';
            $dotgoogle_authenticator = '0';
            $dotgoogle_authenticator_png = '0';
            $ret = $CI->serverScriptHelper->shell("/usr/sausalito/sbin/gauth_check.sh $loginName" . "$is_there", $output, 'root', $BX_SESSION['sessionId']);
            if ($ret == 0) {
                // Get Google Authenticator Config:
                $output = '';
                $full_path_to_dotgoogle_authenticator = "~$loginName/.google_authenticator";
                $tempname = tempnam("/var/cache/admserv/", $loginName . "-google_authenticator_");
                $ret = $CI->serverScriptHelper->shell("/bin/cat $full_path_to_dotgoogle_authenticator " . '|/bin/grep -v ^\" >' . "$tempname", $output, 'root', $BX_SESSION['sessionId']);
                if ($ret == 0) {
                    $dotgoogle_authenticator_content = file_get_contents($tempname);
                    $ret = $CI->serverScriptHelper->shell("/bin/rm -f $tempname", $res, 'root', $BX_SESSION['sessionId']);
                    $dotgoogle_authenticator = '1';
                }

                // Get Google Authenticator PNG:
                $full_path_to_dotgoogle_authenticator_png = "~$loginName/.google_authenticator.png";
                $dotgoogle_authenticator_png = '1';
                $output = '';
                $tempnamePNG = tempnam("/var/cache/admserv/", $loginName . "-google_authenticator_") . '.png';
                $ret = $CI->serverScriptHelper->shell("/bin/cp $full_path_to_dotgoogle_authenticator_png  $tempnamePNG", $output, 'root', $BX_SESSION['sessionId']);
                if ($ret == 0) {
                        $ret = $CI->serverScriptHelper->shell("/bin/chown admserv:admserv $tempnamePNG", $nfk, 'root', $BX_SESSION['sessionId']);
                        $dotgoogle_authenticator_png_content = file_get_contents($tempnamePNG);
                        $ret = $CI->serverScriptHelper->shell("/bin/rm -f $tempnamePNG", $res, 'root', $BX_SESSION['sessionId']);
                        $dotgoogle_authenticator_png = '1';
                }
            }
        }

        if ($dotgoogle_authenticator == '0') {
            // Simple Checkbox:
            $GoogleAuthentication_Field = $factory->getBoolean('GoogleAuthentication', $User_SSH['GoogleAuthentication'], $GA_access);
            $container->addFormField(
                $GoogleAuthentication_Field,
                $factory->getLabel('GoogleAuthentication'),
                $pageIds[0]
                );
        }
        else {
            // Show the complex getMultiChoice() with all details and the show/hide button:
            $ga_enabledField = $factory->getMultiChoice('GoogleAuthentication');
            $enable = $factory->getOption('GoogleAuthentication', $ga_is_enabled, 'rw');
            $ga_enabled = $factory->getLabel('GoogleAuthentication', false);
            $enable->setLabel($ga_enabled);
            $ga_enabledField->addOption($enable);
            $ga_enabledField->setHideOptions(TRUE);

            // Authenticator Key:
            $dotgoogle_authenticator_content_array = explode(PHP_EOL, $dotgoogle_authenticator_content);

            $ga_key_data = '';
            if (isset($dotgoogle_authenticator_content_array[0])) {
                $ga_key_data = $dotgoogle_authenticator_content_array[0];
            }
            $ga_keyField = $factory->getTextField('ga_key', $ga_key_data, 'r');
            $ga_keyField->setOptional(TRUE);
            $enable->addFormField($ga_keyField, $factory->getLabel('ga_key'));

            // One-Time Codes:
            $x = array_shift($dotgoogle_authenticator_content_array);
            $otcField = $factory->getTextList("one_time_codes", $cce->array_to_scalar($dotgoogle_authenticator_content_array), 'rw');
            $otcField->setOptional(TRUE);
            $enable->addFormField($otcField, $factory->getLabel('one_time_codes'));

            if ($dotgoogle_authenticator_png == '1') {
                // Authenticator image:
                $GAUTHimg = $factory->getHTMLField("GAUTHimg", '<img src="' . $this->data_uri($dotgoogle_authenticator_png_content,'image/png') . '">');
                $enable->addFormField($GAUTHimg, $factory->getLabel('GAUTHimg'));
            }

            // Labels and Descriptions are from base-ssh, set the separately:
            $BxPage->setLabel('ga_key', $i18n->get("[[base-ssh.ga_key]]"), $i18n->get("[[base-ssh.ga_key_help]]"));
            $BxPage->setLabel('one_time_codes', $i18n->get("[[base-ssh.one_time_codes]]"), $i18n->get("[[base-ssh.one_time_codes_help]]"));
            $BxPage->setLabel('GAUTHimg', $i18n->get("[[base-ssh.GAUTHimg]]"), $i18n->get("[[base-ssh.GAUTHimg_help]]"));

            // Out with the enabler:
            $container->addFormField($ga_enabledField, $factory->getLabel('GoogleAuthentication'), $pageIds[0]);
        }
    }

    function handle(&$ServerScriptHelper, &$errors, &$cce_info, &$attributes) {

        $this->attributes = $attributes;

        if (($attributes['Shell_enabled'] != "0") && ($attributes['Shell_enabled'] != "1") && ($attributes['Shell_enabled'] != "2") && ($attributes['Shell_enabled'] != "3")) {
            if (!isset($this->attributes['Shell_enabled'])) {
                $this->attributes['Shell_enabled'] = "0";
            }
            else {
                // Reverse Map to get the numerical value for 'Shell_enabled':
                $Shell_enabled_Map_Reversed = 
                    array(
                        "none" => "0", 
                        "jailed_sftp_scp_rsync" => "1", 
                        "jailed_shell" => "2", 
                        "full_shell" => "3"
                    );

                $attributes['Shell_enabled'] = $Shell_enabled_Map_Reversed[$attributes['Shell_enabled']];
            }
        }

        $cce = $ServerScriptHelper->getCceClient();
        $cce->set($cce_info["CCE_OID"], 'Shell', array('enabled' => $attributes['Shell_enabled']));
        $errors = array_merge($errors, $cce->errors());

        $cce->set($cce_info["CCE_OID"], 'SSH', array('GoogleAuthentication' => $attributes['GoogleAuthentication']));
        $errors = array_merge($errors, $cce->errors());
    }

    public function data_uri($rawdata, $mime)  {  
        $base64   = base64_encode($rawdata); 
        return ('data:' . $mime . ';base64,' . $base64);
    }

}
?>
